-- stylua: ignore start
import 'CoreLibs/animation.lua'
import 'CoreLibs/animator.lua'
import 'CoreLibs/easing.lua'
import 'CoreLibs/graphics.lua'
import 'CoreLibs/object.lua'
import 'CoreLibs/timer.lua'
import 'CoreLibs/ui.lua'
import 'CoreLibs/utilities/where.lua'
-- stylua: ignore end

import("../lib/tiny.lua")
import("tiny-tools.lua")
import("assets.lua")
import("systems/filter-types.lua")
import("systems/collision-detection.lua")
import("systems/collision-resolution.lua")
import("systems/draw.lua")
import("systems/gravity.lua")
import("systems/velocity.lua")
import("ingredients/ingredients.lua")
import("cart.lua")

local tiny <const> = tiny
local gfx <const> = playdate.graphics
playdate.display.setRefreshRate(50)
gfx.setBackgroundColor(gfx.kColorWhite)

cart = Cart.new()
local floorSize = { x = 10000, y = 10 }
floor = {
    position = { x = 0, y = 235 },
    size = floorSize,
    canBounce = {
        flat = { x = 0, y = 0 },
        mult = { x = 0.9, y = -0.5 },
    },
    drawAsRectangle = { size = floorSize },
    isSolid = true,
}

Camera = {
    --- The upper-left corner of the Camera is represented by this `pan` value
    pan = {
        x = 0,
        y = 0,
    },
}

Score = {
    points = 0,
}

-- TODO: Shops with random upgrades instead of fixed ones.

function Score:add(add)
    self.points = self.points + add
end

function Score:draw()
    gfx.drawText("Z|" .. self.points, 360, 5)
end

world = tiny.world(
    fallSystem,
    velocitySystem,
    collisionResolution,
    collisionDetection,

    drawRectanglesSystem,
    drawSpriteSystem,

    cart,
    floor
)

local ingredientsEveryX = 50

local function init()
    for i = 1, Ingredients.cacheSize() do
        world:addEntity(Ingredients.getNext(i * ingredientsEveryX + (math.random(0, 20)), math.random(50, 200)))
    end
end

init()

function playdate.update()
    local deltaSeconds = playdate.getElapsedTime()
    playdate.resetElapsedTime()
    gfx.clear(gfx.kColorWhite)
    playdate.drawFPS(5, 5)

    Camera.pan.x = math.max(0, cart.position.x - 200)
    Camera.pan.y = math.min(0, cart.position.y - 120)

    local newestX = Ingredients.newestIngredient().position.x
    local panX = Camera.pan.x
    local rightEdge = panX + 400
    local offset = 600
    while newestX < (rightEdge + 100) do
        newestX = newestX + ingredientsEveryX
        world:addEntity(Ingredients.getNext(newestX, math.random(-500, 150)))
    end
    floor.position.x = Camera.pan.x - offset
    gfx.setDrawOffset(-Camera.pan.x, -Camera.pan.y)

    world:update(deltaSeconds)

    gfx.setDrawOffset(0, 0)
    Score:draw()

    if playdate.buttonJustPressed(playdate.kButtonA) then
        Cart.reset(cart)
        init()
    end
end
